@@ -1,13 +1,14 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
-# (C) 2015 Clark Williams <williams@redhat.com>
+# (C) 2018,2019 Clark Williams <williams@redhat.com>
+# (C) 2015,2016 Clark Williams <williams@redhat.com>
 # (C) 2009 Clark Williams <williams@redhat.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License Version 2
 # as published by the Free Software Foundation.
 
-from __future__ import print_function
+
 
 import sys
 import os
@@ -16,7 +17,7 @@ import subprocess
 import errno
 import os.path
 
-version = "0.6"
+version = "0.8"
 debugging = False
 quiet = False
 watch = False
@@ -83,7 +84,7 @@ class DebugFS(object):
             try:
                 val = f.readline()
             except OSError as e:
-                print ("errno: %s" % e)
+                print("errno: %s" % e)
                 if e.errno == errno.EAGAIN:
                     val = None
                 else:
@@ -106,6 +107,13 @@ class DebugFS(object):
     def getpath(self, item):
         return os.path.join(self.mountpoint, item)
 
+# Exception class for when tracer is not available
+class DetectorNotAvailable(Exception):
+    def __init__(self, name, msg):
+        self.args = (name, msg)
+        self.name = name
+        self.msg = msg
+
 #
 # Class used to manage loading and unloading of the
 # hwlat kernel module. Like the debugfs class
@@ -113,87 +121,84 @@ class DebugFS(object):
 # leave it alone when cleaning up.
 #
 class Kmod(object):
-    ''' class to manage loading and unloading hwlat.ko'''
+    ''' class to manage loading and unloading of kernel modules'''
 
     names = ("hwlat_detector", "smi_detector")
+
     def __check_builtin(self):
         for l in open(os.path.join('/lib/modules', os.uname()[2], 'modules.builtin'), "r"):
-            for m in Kmod.names:
-                if m in l:
-                    debug("found %s as builtin" % m)
-                    return m
-        return None
+            if self.name in l:
+                debug("found %s as builtin" % self.namename)
+                return True
+        return False
 
-    def __find_modname(self):
-        debug("looking for modules")
+    def __find_module(self):
+        debug("looking for module %s" % self.name)
         path = os.path.join("/lib/modules",
                             os.uname()[2],
                             "kernel/drivers/misc")
         debug("module path: %s" % path)
-        for m in Kmod.names:
-            mpath = os.path.join(path, m) + ".ko"
-            debug("checking %s" % mpath)
-            if os.path.exists(mpath):
-                return m
-        raise RuntimeError("no detector module found!")
+        mpath = os.path.join(path, self.name) + ".ko"
+        debug("checking %s" % mpath)
+        if os.path.exists(mpath):
+            return True
+        return False
 
-    def __init__(self):
+    def __init__(self, name):
+        if name not in Kmod.names:
+            raise RuntimeError("unsupported module name: %s" % name)
+        if name == "smi_detector":
+            raise RuntimeError("smi_detector module no longer supported!")
+        self.name = name
         self.preloaded = False
         self.builtin = False
 
         # check for builtin
-        self.modname = self.__check_builtin()
-        if self.modname:
+        if self.__check_builtin():
             self.builtin = True
             return
 
-        # now look for module
-        f = open ('/proc/modules')
-        for l in f:
+        # now look for already loaded module
+        for l in open('/proc/modules'):
             field = l.split()
-            if field[0] in Kmod.names:
+            if self.name in field[0]:
                 self.preloaded = True
-                self.modname = field[0]
-                debug("using already loaded %s" % self.modname)
-                f.close()
-                break
-        f.close()
-        self.modname = self.__find_modname()
+                debug("using already loaded %s" % self.name)
+                return
+        if not self.__find_module():
+            raise DetectorNotAvailable(name, "module %s does not exist!" % self.name)
 
     def load(self):
         if self.builtin:
-            debug("not loading %s (builtin)" % self.modname)
+            debug("not loading %s (builtin)" % self.name)
             return True
         if self.preloaded:
-            debug("not loading %s (already loaded)" % self.modname)
+            debug("not loading %s (already loaded)" % self.name)
             return True
-        cmd = ['/sbin/modprobe', self.modname]
+        cmd = ['/sbin/modprobe', self.name]
         return (subprocess.call(cmd) == 0)
 
     def unload(self):
         if self.preloaded or self.builtin:
-            debug("Not unloading %s" % self.modname)
+            debug("Not unloading %s" % self.name)
             return True
-        cmd = ['/sbin/modprobe', '-r', self.modname]
+        cmd = ['/sbin/modprobe', '-r', self.name]
         return (subprocess.call(cmd) == 0)
 
 #
-# wrapper class for detection modules
+# base class for detection modules
 #
 class Detector(object):
-    '''wrapper class for managing detector modules'''
+    '''base class for detector modules'''
     def __init__(self):
+        self.type = "unknown"
         if os.getuid() != 0:
             raise RuntimeError("Must be root")
         self.debugfs = DebugFS()
-        self.kmod = Kmod()
-        self.setup()
-        if self.kmod.modname == "hwlat_detector":
-            self.detector = Hwlat(self.debugfs)
-        elif self.kmod.modname == "smi_detector":
-            self.detector = Smi(self.debugfs)
+        if not self.debugfs.mount():
+            raise RuntimeError("failed to mount debugfs")
         self.samples = []
-        self.testduration = 10 # ten seconds
+        self.testduration = 30 # ten seconds
         self.have_msr = False
         self.initsmi = []
         if os.path.exists('/usr/sbin/rdmsr'):
@@ -205,81 +210,197 @@ class Detector(object):
         if self.have_msr:
             p = subprocess.Popen(['/usr/sbin/rdmsr', '-a', '-d', '0x34'], stdout=subprocess.PIPE)
             p.wait()
-            counts = [ int(x.strip()) for x in p.stdout.readlines()]
+            counts = [int(x.strip()) for x in p.stdout.readlines()]
         return counts
 
-    def force_cleanup(self):
-        debug("forcing unload of hwlat module")
-        self.kmod.preloaded = False
-        debug("forcing unmount of debugfs")
-        self.debugfs.premounted = False
-        self.cleanup()
-        debug("exiting")
-        sys.exit(0)
+    # methods for preventing/enabling c-state transitions
+    # openinging /dev/cpu_dma_latency and writeing a 32-bit zero to that file will prevent
+    # c-state transitions while the file descriptor is open.
+    # use c_states_off() to disable c-state transitions
+    # use c_states_on() to close the file descriptor and re-enable c-states
+    #
+    def c_states_off(self):
+        self.dma_latency_handle = os.open("/dev/cpu_dma_latency", os.O_WRONLY)
+        os.write(self.dma_latency_handle, b'\x00\x00\x00\x00')
+        debug("c-states disabled")
 
-    def setup(self):
-        if not self.debugfs.mount():
-            raise RuntimeError("Failed to mount debugfs")
-        if not self.kmod.load():
-            raise RuntimeError("Failed to unload hwlat")
+    def c_states_on(self):
+        if self.dma_latency_handle:
+            os.close(self.dma_latency_handle)
+        debug("c-states enabled")
 
     def cleanup(self):
-        if not self.kmod.unload():
-            raise RuntimeError("Failed to unload hwlat")
-        if not self.debugfs.umount():
-            raise RuntimeError("Failed to unmount debugfs")
+        raise RuntimeError("must override base method 'cleanup'!")
 
     def get(self, field):
-        return self.detector.get(field)
+        '''get the value of a debugfs field'''
+        raise RuntimeError("must override base method 'get'!")
 
     def set(self, field, val):
-        return self.detector.set(field, val)
+        '''set a value in a debugfs field'''
+        raise RuntimeError("must override base method 'set'!")
+
+    def save(self, reportfile=None):
+        '''save sample data to reportfile'''
+        raise RuntimeError("must override base method 'save'!")
+
+    def display(self):
+        '''output the sample data as a string'''
+        raise RuntimeError("must override base method 'display'!")
 
     def start(self):
         count = 0
-        threshold = self.get("threshold")
+        threshold = int(self.get("threshold"))
+        self.c_states_off()
         debug("enabling detector module (threshold: %d)" % threshold)
-        self.detector.set("enable", 1)
-        debug("first attempt at enable")
-        while self.detector.get("enable") == 0:
+        self.set("enable", 1)
+        while self.get("enable") == 0:
             debug("still disabled, retrying in a bit")
             count += 1
             time.sleep(0.1)
             debug("retrying enable of detector module (%d)" % count)
-            self.detector.set("enable", 1)
+            self.set("enable", 1)
         if self.get("threshold") != threshold:
             debug("start: threshold reset by start, fixing")
             self.set("threshold", threshold)
-        debug("detector module enabled (threshold: %d)" % self.get("threshold"))
+        debug("detector module enabled (threshold: %d)" % int(self.get("threshold")))
 
     def stop(self):
         count = 0
         debug("disabling detector module")
-        self.detector.set("enable", 0)
-        debug("first attempt at disable");
-        while self.detector.get("enable") == 1:
+        self.set("enable", 0)
+        while self.get("enable") == 1:
             debug("still enabled, retrying in a bit")
             count += 1
             time.sleep(0.1)
             debug("retrying disable of detector module(%d)" % count)
-            self.detector.set("enable", 0)
+            self.set("enable", 0)
+        self.c_states_on()
         debug("detector module disabled")
 
     def detect(self):
-        debug("Starting hardware latency detection for %d seconds" % self.testduration)
+        '''get detector output'''
+        raise RuntimeError("must override base method 'detect'!")
+#
+# class to handle running the hwlat tracer module of ftrace
+#
+class Tracer(Detector):
+    '''class to wrap access to ftrace hwlat tracer'''
+    __field_translation = {
+        'width'     : "hwlat_detector/width",
+        'window'    : "hwlat_detector/window",
+        'enable'    : "tracing_on",
+        'threshold' : "tracing_thresh",
+    }
+
+    class Sample(object):
+        'private class for tracer sample data'
+        __slots__ = 'timestamp', 'inner', 'outer',
+        def __init__(self, line):
+            fields = line.split()
+            i,o = fields[6].split('/')
+            ts = fields[7][3:]
+            self.timestamp = str(ts)
+            self.inner = int(i)
+            self.outer = int(o)
+
+        def __str__(self):
+            return "ts: %s, inner:%d, outer:%d" % (self.timestamp, self.inner, self.outer)
+
+        def display(self):
+            print(str(self))
+
+        def largest(self):
+            if self.inner > self.outer:
+                return self.inner
+            return self.outer
+
+    def translate(self, field):
+        path = self.debugfs.getpath('tracing')
+        if field not in Tracer.__field_translation:
+            return os.path.join(path, field)
+        return os.path.join(path, Tracer.__field_translation[field])
+
+    def __init__(self):
+        super(Tracer, self).__init__()
+        path = self.debugfs.getpath('tracing/hwlat_detector')
+        if not os.path.exists(path):
+            raise DetectorNotAvailable("hwlat", "hwlat tracer not available")
+        self.type = "tracer"
+        self.samples = []
+        self.set("enable", 0)
+        self.set('current_tracer', 'hwlat')
+
+    def set(self, field, val):
+        path = self.translate(field)
+        self.debugfs.putval(path, str(val))
+
+    def get(self, field):
+        if field == "count":
+            return len(self.samples)
+        elif field == "max":
+            max = 0
+            for values in self.samples:
+                s = int(values.largest())
+                if s > max:
+                    max = s
+            return max
+        return self.debugfs.getval(self.translate(field))
+
+    def detect(self):
+        self.samples = []
+        testend = time.time() + self.testduration
+        pollcnt = 0
         self.start()
         try:
-            self.samples = self.detector.detect(self.testduration)
-        finally:
-            self.stop()
-        debug("Hardware latency detection done (%d samples)" % len(self.samples))
+            while time.time() < testend:
+                pollcnt += 1
+                val = self.get_sample()
+                while val:
+                    self.samples.append(val)
+                    if watch: val.display()
+                    val = self.get_sample()
+                time.sleep(0.1)
+        except KeyboardInterrupt as e:
+            print("interrupted")
+        self.stop()
+        return self.samples
+
+    def get_sample(self):
+        val = None
+        line = self.debugfs.getval("tracing/trace_pipe", nonblocking=True)
+        if line:
+            val = self.Sample(line)
+        return val
+
+    def save(self, output=None):
+        if output:
+            f = open(output, "w")
+            for s in self.samples:
+                f.write("%s\n" % str(s))
+            print("report saved to %s (%d samples)" % (output, len(self.samples)))
+
+    def display(self):
+        for s in self.samples:
+            s.display()
+
+    def cleanup(self):
+        self.set("tracing_on", "0")
+        self.set("current_tracer", "nop")
+        if not self.debugfs.umount():
+            raise RuntimeError("Failed to unmount debugfs")
+
+
 #
 # Class to simplify running the hwlat kernel module
 #
-class Hwlat(object):
+class Hwlat(Detector):
     '''class to wrap access to hwlat debugfs files'''
-    def __init__(self, debugfs):
-        self.debugfs = debugfs
+    def __init__(self):
+        super(Hwlat, self).__init__()
+        self.kmod = Kmod("hwlat_detector")
+        self.type = "kmodule"
+        self.kmod.load()
 
     def get(self, field):
         return int(self.debugfs.getval(os.path.join("hwlat_detector", field)))
@@ -292,10 +413,11 @@ class Hwlat(object):
     def get_sample(self):
         return self.debugfs.getval("hwlat_detector/sample", nonblocking=True)
 
-    def detect(self, duration):
+    def detect(self):
         self.samples = []
-        testend = time.time() + duration
+        testend = time.time() + self.testduration
         pollcnt = 0
+        self.start()
         try:
             while time.time() < testend:
                 pollcnt += 1
@@ -304,115 +426,29 @@ class Hwlat(object):
                     val = val.strip()
                     self.samples.append(val)
                     if watch: print(val)
-                    debug("got a latency sample: %s" % val)
                     val = self.get_sample()
                 time.sleep(0.1)
         except KeyboardInterrupt as e:
             print("interrupted")
-            sys.exit(1)
-        return self.samples
-#
-# the old smi_detector.ko module has different debugfs entries than the modern
-# hwlat_detector.ko module; this object translates the current entries into the
-# old style ones. The only real issue is that the smi_detector module doesn't
-# have the notion of width/window, it has the sample time and the interval
-# between samples. Of course window == sample time + interval, but you have to
-# have them both to calculate the window.
-#
-
-class Smi(object):
-    '''class to wrap access to smi_detector debugfs files'''
-    field_translate = {
-        "count" : "smi_count",
-        "enable" : "enable",
-        "max" : "max_sample_us",
-        "sample" : "sample_us",
-        "threshold" : "latency_threshold_us",
-        "width" : "ms_per_sample",
-        "window" : "ms_between_sample",
-        }
-
-    def __init__(self, debugfs):
-        self.width = 0
-        self.window = 0
-        self.debugfs = debugfs
-
-    def __get(self, field):
-        return int(self.debugfs.getval(os.path.join("smi_detector", field)))
-
-    def __set(self, field, value):
-        debug("__set: %s <-- %d" % (field, value))
-        self.debugfs.putval(os.path.join("smi_detector", field), str(value))
-        if self.__get(field) != value:
-            raise RuntimeError("Error setting %s to %d (%d)" % (field, value, self.__get(field)))
-
-    def get(self, field):
-        name = Smi.field_translate[field]
-        if name != field:
-            debug("get: %s translated to %s" % (field, name))
-        if field == "window":
-            return self.get_window()
-        elif field == "width":
-            return ms2us(self.__get(name))
-        else:
-            return self.__get(name)
-
-    def get_window(self):
-        sample = ms2us(self.__get('ms_per_sample'))
-        interval = ms2us(self.__get('ms_between_samples'))
-        return sample + interval
-
-    def set_window(self, window):
-        width = ms2us(int(self.__get('ms_per_sample')))
-        interval = window - width
-        if interval <= 0:
-            raise RuntimeError("Smi: invalid width/interval values (%d/%d (%d))" % (width, interval, window))
-        self.__set('ms_between_samples', us2ms(interval))
-
-    def set(self, field, val):
-        name = Smi.field_translate[field]
-        if name != field:
-            debug ("set: %s translated to %s" % (field, name))
-        if field == "enable" and val:
-            val = 1
-        if field == "window":
-            self.set_window(val)
-        else:
-            if field == "width":
-                val = us2ms(val)
-            self.__set(name, val)
-
-    def get_sample(self):
-        name = Smi.field_translate["sample"]
-        return self.debugfs.getval(os.path.join('smi_detector', name), nonblocking=True)
-
-    def detect(self, duration):
-        self.samples = []
-        testend = time.time() + duration
-        threshold = self.get("threshold")
-        debug("detect: threshold %d" % threshold)
-        pollcnt = 0
-        try:
-            while time.time() < testend:
-                pollcnt += 1
-                val = self.get_sample()
-                val = val.strip()
-                if int(val) >= threshold:
-                    self.samples.append(val)
-                    if watch: print(val)
-                    debug("got a latency sample: %s (threshold: %d)" % (val, self.get("threshold")))
-                time.sleep(0.1)
-        except KeyboardInterrupt as e:
-            print("interrupted")
-            sys.exit(1)
+        self.stop()
         return self.samples
 
+    def display(self):
+        for s in self.samples:
+            print(s)
 
-def ms2us(val):
-    return val * 1000
+    def save(self, output=None):
+        if output:
+            f = open(output, "w")
+            for s in self.samples:
+                f.write("%s\n" % str(s))
+            print("report saved to %s (%d samples)" % (output, len(self.samples)))
 
-def us2ms(val):
-    return val / 1000
+    def cleanup(self):
+        if not self.kmod.unload():
+            raise RuntimeError("Failed to unload %s" % self.name)
+        if not self.debugfs.umount():
+            raise RuntimeError("Failed to unmount debugfs")
 
 def seconds(str):
     "convert input string to value in seconds"
@@ -462,105 +498,121 @@ def microseconds(str):
     else:
         raise RuntimeError("invalid input for microseconds: '%s'" % str)
 
+#
+# main starts here
+#
+
 if __name__ == '__main__':
-    from optparse import OptionParser
+    from argparse import ArgumentParser
 
-    parser = OptionParser()
-    parser.add_option("--duration", default=None, type="string",
-                      dest="duration",
-                      help="total time to test for hardware latency (<n>{smdw})")
+    parser = ArgumentParser()
+    parser.add_argument("--duration", default=None,
+                        dest="duration",
+                        help="total time to test for hardware latency: <n>{smdw}")
 
-    parser.add_option("--threshold", default=None, type="string",
-                      dest="threshold",
-                      help="value above which is considered an hardware latency")
+    parser.add_argument("--threshold", default=None,
+                        dest="threshold",
+                        help="value above which is considered an hardware latency")
 
-    parser.add_option("--window", default=None, type="string",
-                      dest="window",
-                      help="time between samples")
+    parser.add_argument("--hardlimit", default=None,
+                        dest="hardlimit",
+                        help="value above which the test is considered to fail")
 
-    parser.add_option("--width", default=None, type="string",
-                      dest="width",
-                      help="time to actually measure")
+    parser.add_argument("--window", default=None,
+                        dest="window",
+                        help="time between samples")
 
-    parser.add_option("--report", default=None, type="string",
-                      dest="report",
-                      help="filename for sample data")
+    parser.add_argument("--width", default=None,
+                        dest="width",
+                        help="time to actually measure")
 
-    parser.add_option("--cleanup", action="store_true", default=False,
-                      dest="cleanup",
-                      help="force unload of module and umount of debugfs")
+    parser.add_argument("--report", default=None,
+                        dest="report",
+                        help="filename for sample data")
 
-    parser.add_option("--debug", action="store_true", default=False,
-                      dest="debug",
-                      help="turn on debugging prints")
+    parser.add_argument("--debug", action="store_true", default=False,
+                        dest="debug",
+                        help="turn on debugging prints")
 
-    parser.add_option("--quiet", action="store_true", default=False,
-                      dest="quiet",
-                      help="turn off all screen output")
+    parser.add_argument("--quiet", action="store_true", default=False,
+                        dest="quiet",
+                        help="turn off all screen output")
 
-    parser.add_option("--watch", action="store_true", default=False,
-                      dest="watch",
-                      help="print sample data to stdout as it arrives")
+    parser.add_argument("--watch", action="store_true", default=False,
+                        dest="watch",
+                        help="print sample data to stdout as it arrives")
 
-    (o, a) = parser.parse_args()
+    parser.add_argument("--kmodule", action="store_true", default=False,
+                        dest="kmodule",
+                        help="force using the kernel module")
+
+    args = parser.parse_args()
 
     # need these before creating detector instance
-    if o.debug:
+    if args.debug:
         debugging = True
         quiet = False
         debug("debugging prints turned on")
 
-    if o.quiet:
+    if args.quiet:
         quiet = True
         debugging = False
 
-    detect = Detector()
+    if args.kmodule:
+        detect = Hwlat()
+    else:
+        try:
+            detect = Tracer()
+        except DetectorNotAvailable as err:
+            detect = HwLat()
 
-    if o.cleanup:
-        debug("forcing cleanup of debugfs and hardware latency module")
-        detect.force_cleanup()
-        sys.exit(0)
-
-    if o.threshold:
-        t = microseconds(o.threshold)
+    if args.threshold:
+        t = microseconds(args.threshold)
         detect.set("threshold", t)
         debug("threshold set to %dus" % t)
 
-    if o.window:
-        w = microseconds(o.window)
-        if w < detect.get("width"):
+    if args.hardlimit:
+        hardlimit = microseconds(args.hardlimit)
+    else:
+        hardlimit = int(detect.get("threshold"))
+    debug("hardlimit set to %dus" % hardlimit)
+
+    if args.window:
+        w = microseconds(args.window)
+        if w < int(detect.get("width")):
             debug("shrinking width to %d for new window of %d" % (w/2, w))
             detect.set("width", w/2)
         debug("window parameter = %d" % w)
         detect.set("window", w)
         debug("window for sampling set to %dus" % w)
 
-    if o.width:
-        w = microseconds(o.width)
-        if w > detect.get("window"):
+    if args.width:
+        w = microseconds(args.width)
+        if w > int(detect.get("window")):
             debug("widening window to %d for new width of %d" % (w*2, w))
             detect.set("window", w*2)
         debug("width parameter = %d" % w)
         detect.set("width", w)
         debug("sample width set to %dus" % w)
 
-    if o.duration:
-        detect.testduration = seconds(o.duration)
+    if args.duration:
+        detect.testduration = seconds(args.duration)
     else:
         detect.testduration = 120 # 2 minutes
     debug("test duration is %ds" % detect.testduration)
 
-    if o.watch:
+    if args.watch:
         watch = True
 
-    reportfile = o.report
+    reportfile = args.report
 
     info("hwlatdetect:  test duration %d seconds" % detect.testduration)
+    info("   detector: %s" % detect.type)
     info("   parameters:")
-    info("        Latency threshold: %dus" % detect.get("threshold"))
-    info("        Sample window:     %dus" % detect.get("window"))
-    info("        Sample width:      %dus" % detect.get("width"))
-    info("     Non-sampling period:  %dus" % (detect.get("window") - detect.get("width")))
+    info("        Latency threshold: %dus" % int(detect.get("threshold")))
+    info("        Sample window:     %dus" % int(detect.get("window")))
+    info("        Sample width:      %dus" % int(detect.get("width")))
+    info("     Non-sampling period:  %dus" % (int(detect.get("window")) - int(detect.get("width"))))
     info("        Output File:       %s" % reportfile)
     info("\nStarting test")
 
@@ -568,9 +620,15 @@ if __name__ == '__main__':
 
     info("test finished")
 
-    exceeding = detect.get("count")
-    info("Max Latency: %dus" % detect.get("max"))
+    max_latency = detect.get("max")
+    if max_latency == 0:
+        info("Max Latency: Below threshold")
+    else:
+        info("Max Latency: %dus" % max_latency)
+
     info("Samples recorded: %d" % len(detect.samples))
+
+    exceeding = detect.get("count")
     info("Samples exceeding threshold: %d" % exceeding)
 
     if detect.have_msr:
@@ -583,17 +641,13 @@ if __name__ == '__main__':
                 print("%d SMIs occured on cpu %d" % (smis, i))
         info("SMIs during run: %d" % total_smis)
 
+    maxlatency = int(detect.get("max"))
+
     if reportfile:
-        count = 0
-        f = open(reportfile, "w")
-        for s in detect.samples:
-            count += 1
-            f.write("%s\n" % s)
-        f.close()
-        info("sample data (%d samples) written to %s" % (count, reportfile))
-    else:
-        for s in detect.samples:
-            print("%s" % s)
+        detect.save(reportfile)
+
+    if not watch:
+        detect.display()
 
     detect.cleanup()
-    sys.exit(exceeding)
+    sys.exit(maxlatency > hardlimit)
